// Import polyfills first
import "./config/polyfills";

// Early OAuth redirect handler - must run before any React code
(function handleEarlyOAuthRedirect() {
  // Only run on localhost
  const isLocalhost =
    window.location.hostname === "localhost" ||
    window.location.hostname === "127.0.0.1";

  if (!isLocalhost) return;

  // Check for OAuth code parameter
  const urlParams = new URLSearchParams(window.location.search);
  const oauthCode = urlParams.get("code");

  if (oauthCode) {
    console.log("Early detection of OAuth code on localhost:", oauthCode);

    // Redirect to production domain with the same path and query parameters
    const productionUrl = `https://reedai.xyz${window.location.pathname}${window.location.search}`;
    console.log("Redirecting to production URL:", productionUrl);

    // Use replace to avoid adding to browser history
    window.location.replace(productionUrl);

    // Prevent further script execution
    throw new Error("REDIRECT_IN_PROGRESS");
  }
})();

// Console management
if (import.meta.env.PROD) {
  // Store original console methods
  const originalConsole = {
    log: console.log,
    warn: console.warn,
    error: console.error,
  };

  // Limit non-error console messages in production
  console.log = (...args) => {
    // Only log critical messages in production
    if (
      args[0] &&
      typeof args[0] === "string" &&
      (args[0].includes("CRITICAL") ||
        args[0].includes("ERROR") ||
        args[0].includes("FATAL"))
    ) {
      originalConsole.log(...args);
    }
  };

  console.warn = (...args) => {
    if (
      args[0] &&
      typeof args[0] === "string" &&
      (args[0].includes("CRITICAL") ||
        args[0].includes("ERROR") ||
        args[0].includes("FATAL"))
    ) {
      originalConsole.warn(...args);
    }
  };
}

import * as Sentry from "@sentry/react";
import { Suspense } from "react";
import * as ReactDOMClient from "react-dom/client";
import App from "./App.jsx";
import { ErrorUI, LoadingSpinner2 } from "./components";
import { TooltipProvider } from "./components/ui/tooltip";
import "./index.css";
import "./styles/icon-styles.css";
const { createRoot } = ReactDOMClient;

// ========================================================
// [index] | This is the entry point for the application
// =========================================================

const performanceMonitor = {
  start: () => {
    performance.mark("app-start");
    if (window.perfMetrics) {
      window.perfMetrics.recordMark("app-start");
    }
  },
  end: () => {
    performance.mark("app-end");
    performance.measure("app-startup", "app-start", "app-end");

    const metrics = performance.getEntriesByName("app-startup")[0];
    console.log(`App startup time: ${metrics.duration.toFixed(2)}ms`);

    // Signal that the app is ready
    if (window.markAppReady) {
      window.markAppReady();
    }

    // Remove loading state
    document.documentElement.classList.remove("loading");
    const rootElement = document.getElementById("root");
    if (rootElement) {
      rootElement.classList.add("js-loading-complete");
    }

    // Restore console if we're in a limited mode
    if (window.restoreConsole) {
      setTimeout(window.restoreConsole, 5000);
    }
  },
  measureNetworkActivity: () => {
    // Track network requests
    if (window.performance && performance.getEntriesByType) {
      const resources = performance.getEntriesByType("resource");
      const stats = {
        total: resources.length,
        size: resources.reduce((total, r) => total + (r.transferSize || 0), 0),
        types: {},
      };

      resources.forEach((resource) => {
        const type = resource.initiatorType;
        if (!stats.types[type]) stats.types[type] = 0;
        stats.types[type]++;
      });

      console.log("Network activity:", stats);
    }
  },
};

// Store the root instance so we don't create multiple roots
let root = null;

class AppInitializer {
  constructor(rootElement) {
    // Use the existing root or create a new one
    this.root = root || createRoot(rootElement);
    // Store the root for potential future references
    if (!root) root = this.root;

    this.maxRetries = 3;
    this.baseDelay = 1000;
    this.currentRetry = 0;
  }

  async initialize() {
    try {
      // Start performance monitoring
      performanceMonitor.start();

      // Initialize important features first
      await this.initCriticalFeatures();

      // Render the app
      this.root.render(
        <TooltipProvider>
          <Suspense fallback={<LoadingSpinner2 />}>
            <App />
          </Suspense>
        </TooltipProvider>,
      );

      // End performance monitoring
      performanceMonitor.end();

      // Measure network activity after a slight delay
      setTimeout(() => {
        performanceMonitor.measureNetworkActivity();
      }, 3000);

      // Initialize non-critical features
      this.initDeferredFeatures();
    } catch (error) {
      console.error("Error initializing application:", error);
      await this.handleError();
    }
  }

  async initCriticalFeatures() {
    // Initialize features that are needed for the initial render
    Sentry.init({
      dsn: "https://460365b207e418aa4c75015aa3e3ebc7@o4508968030306304.ingest.us.sentry.io/4508968044658688",
    });
    // This keeps the critical path short
    return Promise.resolve();
  }

  initDeferredFeatures() {
    // These features are initialized after the app is rendered
    setTimeout(() => {
      // Initialize analytics only in production and if the file exists
      if (import.meta.env.PROD) {
        import("./services/analytics.js").catch((e) => {
          console.warn("Analytics module not available:", e);
        });
      }

      // Pre-load commonly accessed routes if they exist
      const preloadRoutes = async () => {
        try {
          // Dynamic imports removed for archived pages
          console.info("Preloading routes disabled for archived pages");
        } catch (error) {
          // This catch will likely never be called due to the individual catch handlers
          console.warn("Error preloading routes:", error);
        }
      };

      // Execute route preloading during idle time
      if (window.requestIdleCallback) {
        window.requestIdleCallback(preloadRoutes);
      } else {
        setTimeout(preloadRoutes, 2000);
      }

      // Register service worker in production
      if ("serviceWorker" in navigator && import.meta.env.PROD) {
        navigator.serviceWorker.register("/sw.js").catch((error) => {
          console.warn("ServiceWorker registration failed:", error);
        });
      }
    }, 2000);
  }

  async handleError() {
    if (this.currentRetry < this.maxRetries) {
      this.currentRetry++;
      const delay = this.baseDelay * this.currentRetry; // Exponential backoff

      console.log(
        `Retrying in ${delay}ms... (Attempt ${this.currentRetry} of ${this.maxRetries})`,
      );

      await new Promise((resolve) => setTimeout(resolve, delay));
      return this.initialize();
    }

    this.showErrorUI();
  }

  showErrorUI() {
    this.root.render(<ErrorUI onRetry={() => window.location.reload()} />);
  }
}

const startApp = () => {
  const rootElement = document.getElementById("root");

  if (!rootElement) {
    throw new Error("Failed to find the root element");
  }

  const app = new AppInitializer(rootElement);
  app.initialize();
};

// Create a new StripeProvider component file
if (typeof document !== "undefined") {
  // Create the StripeProvider component file
  import("./components/stripe/StripeProvider.jsx").catch((e) => {
    console.error("Failed to preload StripeProvider component:", e);
  });
}

try {
  // Start the app with a small delay to allow the browser to paint the initial UI
  if (document.readyState === "loading") {
    document.addEventListener("DOMContentLoaded", () =>
      setTimeout(startApp, 10),
    );
  } else {
    setTimeout(startApp, 10);
  }
} catch (error) {
  console.error("Critical initialization error:", error);
  document.body.innerHTML = `
    <div class="flex flex-col items-center justify-center h-screen p-4 space-y-4">
      <h1 class="text-xl font-semibold text-red-600">Critical Error</h1>
      <p class="text-gray-600">Unable to initialize application.</p>
    </div>
  `;
}

// Add hot module reloading support
if (import.meta.hot) {
  import.meta.hot.accept();
}
